/*
GLDM - Graphical Liberty Display Manager

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef AUTHPROVIDER_H_INCLUDED
#define AUTHPROVIDER_H_INCLUDED

#include <string>
#include <vector>
/** Every authprovider must be derived from this abstract class.
  */
class AuthProvider
{
    public:
    AuthProvider() : mChildEnv(NULL) {}
    virtual ~AuthProvider() {}


    /** \brief Auth provider specific function for checking login data
      *
      * \return True if provided data is valid.
      */
    virtual bool checkOwnAuth(const std::string& login, const std::string& password) = 0;


    /** \brief Apply environment
      * Sets up environment variables etc.
      */
    virtual void applyEnvironment(const std::string& login, const std::string& displayname);


    /** \brief Drop privileges to specified loginname
      *
      * Drops current user's privileges to specified user.
      *
      * Obviously, it's only dropping of privs if you're root; otherwise it
      * won't even succeed. Thus, it makes no sense in demo mode.
      */
    virtual bool dropPrivileges(const std::string& login);


    /** \todo Document me
      * \todo change into std::strings
      */
    virtual void setClientAuth(const char* login, const char* mcookie, const char* displayname);



    /** \brief Switch currently active user to another one; execute specified command
      *
      * This function should fork(), set up environment, drop privileges, and
      * execute specified command. Setting up environment should be done
      * by retreiving as much environment data by querying data source class
      * [[still unavalable]] for correct environment data.
      *
      * \todo Implement generic method in AuthProvider::switchUser()
      * \todo Provide data source class for environment setup data
      */
    virtual bool switchUser(const std::string& login, const std::string& command, const std::string& displayname, const std::string& sessioncommand);


    /** \brief Returns this a.p.'s users to appear on user list in UI
      * This function will return a list of users that satisfy the
      * criteria to appear on the interface's list of users.
      * Items from this list are then merged into a total list of
      * users which are then returned to the user interface.
      *
      * Criteria can be, for example, having a certain UID on the
      * system (e.g. 1000+), being flagged to appear on the list,
      * not being blacklisted, etc. This depends on the auth provider
      * and eventually on its configuration options.
      *
      * (By default, if not overriden, this returns an empty list.)
      *
      * \return list of usernames
      */
    virtual std::vector<std::string> getOwnLoginListUsers() { return std::vector<std::string> (); }

    /** \todo document me
      */
    virtual std::string getOwnDisplayName(std::string username);

    /** \todo document me
      */
    virtual std::string getOwnHomeDir(std::string username);

    /** \brief Returns a list of all users to appear on user list in UI
      * Combines all users returned by getOwnLoginListUsers() into
      * one complete list.
      *
      * \return list of usernames
      * \sa getOwnLoginListUsers()
      */
    static std::vector<std::string> getLoginListUsers();

    /** \brief Global-scope API for checking login data.
      * Global-scope API for checking validity of provided login data.
      * Will iterate through registered auth providers and ask each
      * one if it allows login.
      *
      * \return First auth provider that says login data is valid. Otherwise NULL.
      */
    static AuthProvider* checkAuth(const std::string& login, const std::string& password);

    /** \brief Registers an auth provider
      * Call this function in order to register the auth provider into the
      * system.
      */
    static void registerProvider(AuthProvider* p);

    /** \todo document me
      */
    static std::string getDisplayName(std::string username);

    /** \todo document me
      */
    static std::string getHomeDir(std::string username);


    private:
    static std::vector<AuthProvider*> sAuthProviders; ///< Contains all auth providers through which we can iterate to check login data.
    char** mChildEnv;

};



#endif
